[lld] r276784 - [ELF] Linkerscript: symbol assignments with indentifiers on the right side of expression.

George Rimar via llvm-commits llvm-commits at lists.llvm.org
Tue Jul 26 11:18:58 PDT 2016


Author: grimar
Date: Tue Jul 26 13:18:58 2016
New Revision: 276784

URL: http://llvm.org/viewvc/llvm-project?rev=276784&view=rev
Log:
[ELF] Linkerscript: symbol assignments with indentifiers on the right side of expression.

In symbol assignments symbol may appear on the right-hand side of the expression:
(https://svnweb.freebsd.org/base/head/sys/conf/ldscript.amd64?revision=284870&view=markup#l8)

kernphys = CONSTANT (MAXPAGESIZE);
 . = kernbase + kernphys + SIZEOF_HEADERS;

Patch implements that.

Differential revision: https://reviews.llvm.org/D22759

Added:
    lld/trunk/test/ELF/linkerscript/linkerscript-symbol-assignexpr.s
Modified:
    lld/trunk/ELF/LinkerScript.cpp

Modified: lld/trunk/ELF/LinkerScript.cpp
URL: http://llvm.org/viewvc/llvm-project/lld/trunk/ELF/LinkerScript.cpp?rev=276784&r1=276783&r2=276784&view=diff
==============================================================================
--- lld/trunk/test/ELF/linkerscript/linkerscript-symbol-assignexpr.s (added)
+++ lld/trunk/test/ELF/linkerscript/linkerscript-symbol-assignexpr.s Tue Jul 26 13:18:58 2016
@@ -0,0 +1,32 @@
+# REQUIRES: x86
+# RUN: llvm-mc -filetype=obj -triple=x86_64-unknown-linux %s -o %t
+
+# RUN: echo "SECTIONS { \
+# RUN:         symbol = CONSTANT(MAXPAGESIZE); \
+# RUN:         symbol2 = symbol + 0x1234; \
+# RUN:         symbol3 = symbol2; \
+# RUN:       }" > %t.script
+# RUN: ld.lld -o %t1 --script %t.script %t
+# RUN: llvm-objdump -t %t1 | FileCheck %s
+
+# CHECK:      SYMBOL TABLE:
+# CHECK-NEXT: 0000000000000000 *UND* 00000000
+# CHECK-NEXT: 0000000000000120 .text 00000000 _start
+# CHECK-NEXT: 0000000000000121 .text 00000000 foo
+# CHECK-NEXT: 0000000000001000 *ABS* 00000000 symbol
+# CHECK-NEXT: 0000000000002234 *ABS* 00000000 symbol2
+# CHECK-NEXT: 0000000000002234 *ABS* 00000000 symbol3
+
+# RUN: echo "SECTIONS { \
+# RUN:         symbol2 = symbol; \
+# RUN:       }" > %t2.script
+# RUN: not ld.lld -o %t2 --script %t2.script %t 2>&1 \
+# RUN:  | FileCheck -check-prefix=ERR %s
+# ERR: symbol not found: symbol
+
+.global _start
+_start:
+ nop
+
+.global foo
+foo:




More information about the llvm-commits mailing list
--- lld/trunk/ELF/LinkerScript.cpp (original)
+++ lld/trunk/ELF/LinkerScript.cpp Tue Jul 26 13:18:58 2016
@@ -756,6 +756,29 @@ SymbolAssignment *ScriptParser::readAssi
 // script expression.
 Expr ScriptParser::readExpr() { return readExpr1(readPrimary(), 0); }
 
+static uint64_t getSymbolValue(StringRef S) {
+  switch (Config->EKind) {
+  case ELF32LEKind:
+    if (SymbolBody *B = Symtab<ELF32LE>::X->find(S))
+      return B->getVA<ELF32LE>();
+    break;
+  case ELF32BEKind:
+    if (SymbolBody *B = Symtab<ELF32BE>::X->find(S))
+      return B->getVA<ELF32BE>();
+    break;
+  case ELF64LEKind:
+    if (SymbolBody *B = Symtab<ELF64LE>::X->find(S))
+      return B->getVA<ELF64LE>();
+    break;
+  case ELF64BEKind:
+    if (SymbolBody *B = Symtab<ELF64BE>::X->find(S))
+      return B->getVA<ELF64BE>();
+    break;
+  }
+  error("symbol not found: " + S);
+  return 0;
+}
+
 // This is a part of the operator-precedence parser. This function
 // assumes that the remaining token stream starts with an operator.
 Expr ScriptParser::readExpr1(Expr Lhs, int MinPrec) {
@@ -844,10 +867,13 @@ Expr ScriptParser::readPrimary() {
     return [](uint64_t Dot) { return alignTo(Dot, Target->PageSize); };
   }
 
   // Parse a number literal
   uint64_t V = 0;
-  if (Tok.getAsInteger(0, V))
-    setError("malformed number: " + Tok);
+  if (Tok.getAsInteger(0, V)) {
+    if (!isValidCIdentifier(Tok))
+      setError("malformed number: " + Tok);
+    return [=](uint64_t Dot) { return getSymbolValue(Tok); };
+  }
   return [=](uint64_t Dot) { return V; };
 }
 
@@ -844,10 +867,13 @@ Expr ScriptParser::readPrimary() {
   }
 
-  // Parse a number literal
+  // Parse a symbol name or a number literal.
   uint64_t V = 0;

Added: lld/trunk/test/ELF/linkerscript/linkerscript-symbol-assignexpr.s
URL: http://llvm.org/viewvc/llvm-project/lld/trunk/test/ELF/linkerscript/linkerscript-symbol-assignexpr.s?rev=276784&view=auto
==============================================================================